#include <stdint.h>

const uint8_t  _USB_MSC_MANUFACTURER_STRING[]  = "MikroElektronika";
const uint8_t  _USB_MSC_PRODUCT_STRING[]       = "Mass Storage by Mikroe";
const uint8_t  _USB_MSC_SERIALNUMBER_STRING[]  = "0x00000002";
const uint8_t  _USB_MSC_CONFIGURATION_STRING[] = "Mass Storage Config Desc string";
const uint8_t  _USB_MSC_INTERFACE_STRING[]     = "Mass Storage mscInterface Desc string";

const uint8_t  _USB_MSC_STR_DESC_SIZE          = 4;
const uint16_t _USB_MSC_STR_DESC_LANGID        = 0x409;

const uint8_t _USB_MSC_CONFIG_DESC_SIZ  = 32;   // Configuration descriptor size
const uint8_t _USB_MSC_PACKET_MAX_SIZE  = 64;   // Max packet size for endpoint
const uint8_t _USB_MSC_IN_EP_NUM  = 1;          // IN endpoint number
const uint8_t _USB_MSC_OUT_EP_NUM = 1;          // OUT endpoint number

//String Descriptor Zero, Specifying Languages Supported by the Device
const uint8_t USB_MSC_LangIDDesc[_USB_MSC_STR_DESC_SIZE] = {
  _USB_MSC_STR_DESC_SIZE,
  _USB_DEV_DESCRIPTOR_TYPE_STRING,
  _USB_MSC_STR_DESC_LANGID & 0xFF,
  _USB_MSC_STR_DESC_LANGID >> 8,
};


// device descriptor
const uint8_t USB_MSC_device_descriptor[] = {
  0x12, // bLength
  0x01, // bDescriptorType
  0x00, // bcdUSB
  0x02,
  0x00, // bDeviceClass
  0x00, // bDeviceSubClass
  0x00, // bDeviceProtocol
  0x40, // bMaxPacketSize0
  0x00, // idVendor
  0x00,
  0x02, // idProduct
  0x00,
  0x00, // bcdDevice
  0x02,
  0x01, // iManufacturer
  0x02, // iProduct
  0x03, // iSerialNumber
  0x01  // bNumConfigurations

};

// Configuration descriptor with all interfaces and endpoints
// descriptors for all of the interfaces
const uint8_t USB_MSC_cfg_descriptor[_USB_MSC_CONFIG_DESC_SIZ] = {
  // Configuration descriptor
  0x09,                                   // bLength: Configuration Descriptor size
  _USB_DEV_DESCRIPTOR_TYPE_CONFIGURATION, // bDescriptorType: Configuration
  _USB_MSC_CONFIG_DESC_SIZ & 0xFF,        // wTotalLength: Bytes returned
  _USB_MSC_CONFIG_DESC_SIZ >> 8,
  0x01,                                   // bNumInterfaces: 1 interface
  0x01,                                   // bConfigurationValue: Configuration value
  0x04,                                   // iConfiguration: Index of string descriptor describing the configuration
  0xC0,                                   // bmAttributes: self powered
  0x32,                                   // MaxPower 100 mA: maximum power consuption of the device

  // Mass Storage interface
  0x09,                                   // bLength: interface descriptor size
  0x04,                                   // bDescriptorType: interface descriptor type
  0x00,                                   // bInterfaceNumber: Number of Interface
  0x00,                                   // bAlternateSetting: Alternate setting
  0x02,                                   // bNumEndpoints: Number of endpoints
  0x08,                                   // bInterfaceClass: MSC
  0x06,                                   // bInterfaceSubClass : SCSI transparent
  0x50,                                   // nInterfaceProtocol
  0x05,                                   // iInterface: Index of string descriptor

  // Mass Storage Endpoints
  0x07,                                   // bLength: Endpoint Descriptor size
  _USB_DEV_DESCRIPTOR_TYPE_ENDPOINT,      // bDescriptorType: endpoint descriptor type

  0x80 | _USB_MSC_IN_EP_NUM,              // bEndpointAddress: Endpoint Address (IN)
  0x02,                                   // bmAttributes: Bulk endpoint
  _USB_MSC_PACKET_MAX_SIZE,               // wMaxPacketSize: maximum packet size for endpoint
  0x00,
  0x00,                                   // bInterval: Polling Interval

  0x07,                                   // bLength: Endpoint Descriptor size
  _USB_DEV_DESCRIPTOR_TYPE_ENDPOINT,      // bDescriptorType: endpoint descriptor type
  _USB_MSC_OUT_EP_NUM,                    // bEndpointAddress: Endpoint Address (OUT)
  0x02,                                   // bmAttributes: Bulk endpoint
  _USB_MSC_PACKET_MAX_SIZE,               // wMaxPacketSize: maximum packet size for endpoint
  0x00,
  0x00                                    // bInterval
};