/*
   Alexander Wolf 
   15 Oktober 2014
 * Project name:
      USB CD - MS switch
 * Description
    uC starts in CD (communication) mode via a virtual Com Port on your computer. 
    When sending the command ;MS; via Terminal from PC to uC, the uC starts into Mass Storage mode and SD Card shows up as USB Stick on your Computer. 
    Remove of USB Cable disables MS mode and turns uC into CD Mode after reattach of USB cable.
    
    It is also possible to switch bewteen both modes using external interrupt on port D0. Pressing the button in CD uC starts into MS mode and otherway round.

 * Test configuration:
     MCU:             P32MX795F512L
     dev.board:       EasyPIC Fusion v7
                      http://www.mikroe.com/easypic-fusion/
     Oscillator:      XT-PLL, 80.000MHz
     Ext. Modules:    None.
     SW:              mikroC PRO for PIC32
                      http://www.mikroe.com/mikroc/pic32/
     Notes:
     check correct USB settings in Project Settings. Insert SD card. Set all Hardware switches on Board correct (Fusion v7 SW 13 & SW14).
     Be patient, Mass Storage takes around 30sec to show up in Windows.
     MikroE USB Library and MikroE Virtual Comport Drivers has to be installed.
     
     For ext Interrupt:
     - SW10 Port D/L on high Button press level switch
     - PORTD/L D0 PULL Down resistor
     
*/

#include <stdio.h>
#include "__Lib_USB_Device.h"

sbit Mmc_Chip_Select           at LATD12_bit;
sbit Mmc_Chip_Select_Direction at TRISD12_bit;

char buffer[64];
char MSC_flag = 0;

extern const uint8_t _USB_CDC_BULK_EP_IN;   // Data interface IN endpoint
extern const uint8_t _USB_CDC_BULK_EP_OUT;  // Data interface OUT endpoint

extern void USBDev_MSCInit();
extern void USBDev_MSCMain();

char start_MassStorageUSB();
char start_CommunicationDeviceUSB();

void USB1Interrupt() iv IVT_USB_1 ilevel 7 ics ICS_SRS{
  USBDev_IntHandler();
}

void USBDev_CDCParamsChanged(){
}


void USBDev_CDCDataReceived(uint16_t size){
  const char stn_out[50] = "switching now to USB Mass Storage mode\x0D";
  USBDev_CDCSetReceiveBuffer(buffer);
  if( strstr(buffer,"MS")  )    // check if command was found in buffer
         MSC_flag = 1;
  USBDev_CDCSendData( stn_out, strlen(stn_out) );
}

char start_MassStorageUSB(){
  delay_ms(1000);
  U1OTGIR   = 0x0000;
  U1OTGIE   = 0x0000;
  U1OTGSTAT = 0x0000;
  U1OTGCON  = 0x0000;
  U1PWRC    = 0x0000;
  U1IR      = 0x0000;
  U1IE      = 0x0000;
  U1EIR     = 0x0000;
  U1EIE     = 0x0000;
  U1STAT    = 0x0000;
  U1CON     = 0x0000;
  U1ADDR    = 0x0000;
  U1BDTP1   = 0x0000;
  U1FRML    = 0x0000;
  delay_ms(1000);
  USBDev_MSCInit();
  USBDev_Init();
  USBIE_bit = 1;
  IPC11bits.USBIP = 7;
  EnableInterrupts();
  delay_ms(100);
  return 1;
}

char start_CommunicationDeviceUSB(){
  delay_ms(1000);
  MSC_flag = 0;
  U1OTGIR   = 0x0000;
  U1OTGIE   = 0x0000;
  U1OTGSTAT = 0x0000;
  U1OTGCON  = 0x0000;
  U1PWRC    = 0x0000;
  U1IR      = 0x0000;
  U1IE      = 0x0000;
  U1EIR     = 0x0000;
  U1EIE     = 0x0000;
  U1STAT    = 0x0000;
  U1CON     = 0x0000;
  U1ADDR    = 0x0000;
  U1BDTP1   = 0x0000;
  U1FRML    = 0x0000;
  delay_ms(1000);
  USBDev_CDCInit();
  USBDev_Init();
  USBIE_bit = 1;
  IPC11bits.USBIP = 7;
  EnableInterrupts();
  while(USBDev_GetDeviceState() != _USB_DEV_STATE_CONFIGURED)   // Wait until device is configured (enumeration is successfully finished)
    ;
  USBDev_CDCSetReceiveBuffer(buffer);
  return 1;
}

void int0_interrupt() iv IVT_EXTERNAL_0 ilevel 4 ics ICS_AUTO {

  if(MSC_flag)
      MSC_flag = 0;
  else
      MSC_flag = 1;

  INT0IF_bit = 0;                    // Reset INT1 flag
}

void init_extInterrupt0(){

   // Extern Interrupt Config ***************************************************
  INT0IP0_bit = 0;                   // Set INT2 interrupt
  INT0IP1_bit = 0;                   // Set interrupt priorities
  INT0IP2_bit = 1;                   // Set inrrupt priority to 4
  INT0IE_bit  = 1;                   // Set interrupt on INT2 (RE9) to be enabled / disabled
//  EnableInterrupts();                // Enable interruts as previously set

}

void main() {

  AD1PCFG = 0xFFFF;
  TRISD   = 0X0001;               // Port D is output, except PortD0, which is Input
  LATD    = 0X0000;
  
  SPI3_Init_Advanced(_SPI_MASTER, _SPI_8_BIT, 8, _SPI_SS_DISABLE, _SPI_DATA_SAMPLE_MIDDLE, _SPI_CLK_IDLE_HIGH, _SPI_ACTIVE_2_IDLE);
  init_extInterrupt0();
  start_CommunicationDeviceUSB();

  while(1)
     {
     if(MSC_flag)
        {
        MSC_flag = 0;
        start_MassStorageUSB();
        while(U1OTGSTATbits.VBUSVD && MSC_flag){
            USBDev_MSCMain();
            WDTCONSET = 0x01;    // Watchdog service
            }
        MSC_flag = 0;
        delay_ms(2000);
        start_CommunicationDeviceUSB();
        }
     }
     WDTCONSET = 0x01;    // Watchdog service
}